<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       http://brewpixel.com
 * @since      1.0.0
 *
 * @package    BrewPixel_Maintenance
 * @subpackage BrewPixel_Maintenance/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    BrewPixel_Maintenance
 * @subpackage BrewPixel_Maintenance/admin
 * @author     Martin Goldie <martin@brewpixel.com>
 */
class BrewPixel_Maintenance_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $brewpixel_maintenance    The ID of this plugin.
	 */
	private $brewpixel_maintenance;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * The email of the main maintenance user.
	 *
	 * @access   private
	 * @var      string $maintenance_user_email The email of the main maintenance user.
	 */
	private $maintenance_user_email;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $brewpixel_maintenance       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 * @param      string    $maintenance_user_email The email of the main maintenance user.
	 */
	public function __construct( $brewpixel_maintenance, $version, $maintenance_user_email ) {

		$this->brewpixel_maintenance = $brewpixel_maintenance;
		$this->version = $version;
		$this->maintenance_user_email = $maintenance_user_email;
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in BrewPixel_Maintenance_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The BrewPixel_Maintenance_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->brewpixel_maintenance, plugin_dir_url( __FILE__ ) . 'css/brewpixel-maintenance-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in BrewPixel_Maintenance_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The BrewPixel_Maintenance_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->brewpixel_maintenance, plugin_dir_url( __FILE__ ) . 'js/brewpixel-maintenance-admin.js', array( 'jquery' ), $this->version, false );

	}

	public function allow_maintenance_user_file_mod_capability(){
		if ( current_user_can( 'manage_options' ) ) {
			$current_user_email = wp_get_current_user()->user_email;
			if ( $this->maintenance_user_email === $current_user_email ) {
				add_filter( 'file_mod_allowed', '__return_true' );
			}
		}
	}

	/**
	 * Hide core update notice/nag from within WordPress admin dashboard
	 * unless maintenance user.
	 */
	public function hide_update_notice_to_all_but_admin_users() {
		if ( current_user_can( 'manage_options' ) ) {
			$current_user_email = wp_get_current_user()->user_email;
			// Override wp-config for auto-updates.
			add_filter( 'automatic_updater_disabled', '__return_true' );
			if ( $this->maintenance_user_email != $current_user_email ) {
				remove_action( 'admin_notices', 'update_nag', 3 );
			}
		}
	}

	public function add_maintenance_user_setting() {
		if ( class_exists( '\BrewPixel\BrewPress\BrewPress' ) ) {
			$brewpress = new \BrewPixel\BrewPress\BrewPress();
			$domain    = $brewpress->get_brewpress();

			/*
			* SETTINGS
			*/
			new \BrewPixel\BrewPress\AdminSetting\Setting(
				[
					'id'    => 'brewpixel_maintenance_settings_section',
					'title' => 'BrewPixel Maintenance Service',
					// 'callback'  => 'setting_section_callback',
					'page'  => 'general',
					'args'  => '<p>The BrewPixel maintenance plugin overrides the default update notices and emails.</p>',
				],
				[
					'id'    => 'brewpixel_maintenance_email',
					'label' => 'Maintenance User Email',
					// 'callback' => [ $this, 'callback_setting_field' ],
					// 'callback'  => 'setting_section_callback',
					'page'  => 'general',
					'args'  => array( 'Do not change unless authorised.' ),
				]
			);
		}
	}
}
