<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       http://brewpixel.com
 * @since      1.0.0
 *
 * @package    BrewPixel_Maintenance
 * @subpackage BrewPixel_Maintenance/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    BrewPixel_Maintenance
 * @subpackage BrewPixel_Maintenance/includes
 * @author     Martin Goldie <martin@brewpixel.com>
 */
class BrewPixel_Maintenance {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      BrewPixel_Maintenance_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $brewpixel_maintenance    The string used to uniquely identify this plugin.
	 */
	protected $brewpixel_maintenance;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * The email of the main maintenance user.
	 *
	 * @access   private
	 * @var      string $maintenance_user_email The email of the main maintenance user.
	 */
	private $maintenance_user_email;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {

		$this->brewpixel_maintenance = 'brewpixel-maintenance';
		$this->version = '1.0.2';
		$this->maintenance_user_email = get_option( 'brewpixel_maintenance_email' );

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - BrewPixel_Maintenance_Loader. Orchestrates the hooks of the plugin.
	 * - BrewPixel_Maintenance_i18n. Defines internationalization functionality.
	 * - BrewPixel_Maintenance_Admin. Defines all hooks for the admin area.
	 * - BrewPixel_Maintenance_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-brewpixel-maintenance-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-brewpixel-maintenance-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-brewpixel-maintenance-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-brewpixel-maintenance-public.php';

		$this->loader = new BrewPixel_Maintenance_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the BrewPixel_Maintenance_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new BrewPixel_Maintenance_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new BrewPixel_Maintenance_Admin( $this->get_brewpixel_maintenance(), $this->get_version(), $this->get_maintenance_user_email() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'admin_head', $plugin_admin, 'hide_update_notice_to_all_but_admin_users', 1 );
		$this->loader->add_action( 'plugins_loaded', $plugin_admin,'allow_maintenance_user_file_mod_capability' );
		$this->loader->add_action( 'plugins_loaded', $plugin_admin, 'add_maintenance_user_setting' );
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new BrewPixel_Maintenance_Public( $this->get_brewpixel_maintenance(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_brewpixel_maintenance() {
		return $this->brewpixel_maintenance;
	}

	/**
	 * The email of the maintenance user.
	 *
	 * @since     1.0.1
	 * @return    string    The email of the main maintenance user.
	 */
	public function get_maintenance_user_email() {
		return $this->maintenance_user_email;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    BrewPixel_Maintenance_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}


	// @TODO: Check if any updates are needed via Cron and email BrewPixel.

	/*
	 * Maintenance Mode
	 */
	/*
	 * @TODO: Check out wp_maintenance()
	 * https://codex.wordpress.org/Function_Reference/wp_maintenance) ##
	 *
	 * ### Description ###
	 * Dies with a maintenance message when conditions are met.
	 *
	 * Checks for a file in the WordPress root directory named ".maintenance". This file will contain the variable $upgrading, set to the time the file was created. If the file was created less than 10 minutes ago, WordPress enters maintenance mode and displays a message.
	 *
	 * The default message can be replaced by using a drop-in (maintenance.php in the wp-content directory).
	 *
	 * ### Usage ###
	 * <?php wp_maintenance( ); ?>
	 *
	 * ### Source File ###
	 * wp_maintenance() is located in wp-includes/load.php
	 *
	 * Start by creating a file in the root of your WordPress install (on level with wp-settings.php) called .maintenance. Note the preceding dot like a .htaccess file; in Linux this is considered a hidden file. In this file add the following code:
	 * <?php $upgrading = time(); ?>
	 * This code will basically cause the maintenance page to display until you remove the .maintenance file. In wp-settings.php there are 2 checks to see if it should display the maintenance page. First, it makes sure that the .maintenance file exists. Second, it checks that the current time minus the time specified by the $upgrading variable is less than 10 minutes. Using the code above will insure that it is always less than 10 minutes since time() - time() == 0. If you want it to display for a certain period of time you would want to use:
	 *
	 * <?php $upgrading = 1234567890; ?>
	 * For your usage you would want to replace 1234567890 with the unix formatted timestamp of the time minus 10 minutes at which you want the maintenance page to stop displaying.
	 *
	 * For example if I wanted the maintenance page to stop displaying at November 14, 2013 at 20:13:00, I would really set the $upgrading variable to November 14, 2013 at 20:03:00. Notice the 03 instead of 13. In unix time this would look like 1384459380. And the code needed for the .maintenance file would be:
	 *
	 * <?php $upgrading = 1384459380; ?>
	 * Take note that if you use a specific time in the .maintenance file and you do not remove the .maintenance file, your users will see your site and not be affected, however in the admin you will see a notice stating, “An automated WordPress update has failed to complete - please attempt the update again now.” Deleting the .maintenance file will remove this notice.
	 *
	 * After wp-settings.php determines whether or not to put the blog into maintenance mode it checks to see if there is a file titled maintenance.php located in WP_CONTENT_DIR which is by default wp-content/.
	 *
	 * Simply create a file at wp-content/maintenance.php containing the code you want to display the for the maintenance page. Below is a sample of code based off of the default maintenance page.
	 *
	 * <?php
	 * $protocol = $_SERVER["SERVER_PROTOCOL"];
	 * if ( 'HTTP/1.1' != $protocol && 'HTTP/1.0' != $protocol )
	 *     $protocol = 'HTTP/1.0';
	 * header( "$protocol 503 Service Unavailable", true, 503 );
	 * header( 'Content-Type: text/html; charset=utf-8' );
	 * ?>
	 * <html xmlns="http://www.w3.org/1999/xhtml">
	 * <body>
	 *     <h1>Briefly unavailable for scheduled maintenance. Check back in a minute.</h1>
	 * </body>
	 * </html>
	 * <?php die(); ?>
	 * Modify as needed, add some css, some images and there you go.
	 *
	 * how to let a user log into the admin and allowing logged in users access to the front end of the site while in maintenance mode.
	 *
	 * It only takes a little bit of extra code in a file called .maintenance in the root of your WordPress installation to conditionally return a time that falls within the logic described in part 1. Now without forther adieu:
	 *
	 * <?php
	 * function is_user_logged_in() {
	 * $loggedin = false;
	 * foreach ( (array) $_COOKIE as $cookie => $value ) {
	 *         if ( stristr($cookie, 'wordpress_logged_in_') )
	 *             $loggedin = true;
	 *     }
	 *     return $loggedin;
	 * }
	 * if ( ! stristr($_SERVER['REQUEST_URI'], '/wp-admin') && ! stristr($_SERVER['REQUEST_URI'], '/wp-login.php') && ! is_user_logged_in() )
	 *     $upgrading = time();
	 * ?>
	 * Just drop the above code in the .maintenance file

	 */

	/*
 * ---------------------------------------------------------------------------*\
 *  #UPDATE CONTROLS AND NOTICES
 * ---------------------------------------------------------------------------
 */

	/*
	// @TODO: Investigate these filters. How best to disable update notifications?
	add_filter('pre_site_transient_update_core','disable_updates_notification');
	add_filter('pre_site_transient_update_plugins','disable_updates_notification');
	add_filter('pre_site_transient_update_themes','disable_updates_notification');
	function disable_updates_notification() {
		global $wp_version;

		return (object) array( 'last_checked'    => time(),
							   'version_checked' => $wp_version
		);
	}
	*/

}
